@@ -5,6 +5,7 @@ pymatrices Package :-
 """
 
 from copy import deepcopy as _dc
+from itertools import permutations as _perm
 from math import sqrt as _sqrt, cos as _cos, sin as _sin, acos as _acos
 
 class matrix:
@@ -29,10 +30,6 @@ class matrix:
 
 		self.__mat = tempMat
 
-	@property
-	def matrix(self):
-		return self.__mat
-
 	@property
 	def order(self):
 		return (self.__rows, self.__cols)
@@ -45,7 +42,7 @@ class matrix:
 			temp = []
 
 			for j in range(order[0]):
-				temp.append(self.matrix[j][i])
+				temp.append(self[j][i])
 
 			res.append(temp)
 
@@ -55,10 +52,10 @@ class matrix:
 	def primaryDiagonalValues(self):
 		res = []
 
-		for i in range(len(self.matrix)):
-			for j in range(len(self.matrix[i])):
+		for i in range(len(self.asList())):
+			for j in range(len(self[i])):
 				if i == j:
-					res.append(self.matrix[i][j])
+					res.append(self[i][j])
 
 		return res
 
@@ -66,29 +63,45 @@ class matrix:
 	def secondaryDiagonalValues(self):
 		order, res = self.order, []
 
-		for i in range(len(self.matrix)):
-			for j in range(len(self.matrix[i])):
+		for i in range(len(self.asList())):
+			for j in range(len(self[i])):
 				if i+j == (order[0]-1):
-					res.append(self.matrix[i][j])
+					res.append(self[i][j])
 
 		return res
 
 	def __repr__(self):
-		neg, minLen, res = False, len(f"{self.matrix[0][0]}"), ""
+		minLen, order, res = len(f"{self[0][0]}"), self.order, "[ "
 
-		for row in self.matrix:
+		for row in self.asList():
 			for val in row:
 				if len(f"{val}") > minLen:
 					minLen = len(f"{val}")
 
-		for row in self.matrix:
-			for val in row:
-				strVal = f"{val}"
-				res += " "*(minLen-len(strVal)) + strVal + " "
+		for i in range(order[0]):
+			for j in range(order[1]):
+				strVal = f"{self[i][j]}"
+
+				if (i == order[0]-1) and (j == order[1]-1):
+					if (i != 0) and (j == 0):
+						res += " "*(minLen+2-len(strVal)) + strVal + " ]"
+					else:
+						res += " "*(minLen-len(strVal)) + strVal + " ]"
+				else:
+					if (i != 0) and (j == 0):
+						res += " "*(minLen+2-len(strVal)) + strVal + " "
+					else:
+						res += " "*(minLen-len(strVal)) + strVal + " "
 			res += "\n"
 
 		return res
 
+	def __getitem__(self, ind):
+		return self.asList()[ind]
+
+	def __setitem__(self, ind, val):
+		self.asList()[ind] = val
+
 	def __add__(self, other):
 		if isinstance(other, matrix):
 			if self.order == other.order:
@@ -97,7 +110,7 @@ class matrix:
 
 				for i in range(row):
 					for j in range(col):
-						sum_elem = self.matrix[i][j] + other.matrix[i][j]
+						sum_elem = self[i][j] + other[i][j]
 						temp.append(sum_elem)
 
 					res.append(temp)
@@ -117,7 +130,7 @@ class matrix:
 
 				for i in range(row):
 					for j in range(col):
-						sum_elem = self.matrix[i][j] - other.matrix[i][j]
+						sum_elem = self[i][j] - other[i][j]
 						temp.append(sum_elem)
 
 					res.append(temp)
@@ -146,7 +159,7 @@ class matrix:
 						sum_val = 0
 
 						for k in range(tOrder[1]):
-							sum_val += (self.matrix[i][k] * T_other.matrix[j][k])
+							sum_val += (self[i][k] * T_other[j][k])
 
 						temp.append(sum_val)
 					res.append(temp)
@@ -159,7 +172,7 @@ class matrix:
 				temp = []
 
 				for col in range(order[1]):
-					temp.append(self.matrix[row][col]*other)
+					temp.append(self[row][col]*other)
 
 				res.append(temp)
 
@@ -173,7 +186,7 @@ class matrix:
 				temp = []
 
 				for col in range(order[1]):
-					temp.append(self.matrix[row][col]/other)
+					temp.append(self[row][col]/other)
 
 				res.append(temp)
 			return matrix(res)
@@ -187,7 +200,7 @@ class matrix:
 			if sOrder == other.order:
 				for row in range(sOrder[0]):
 					for col in range(sOrder[1]):
-						if self.matrix[row][col] != other.matrix[row][col]:
+						if self[row][col] != other[row][col]:
 							return False
 				else:
 					return True
@@ -203,17 +216,20 @@ class matrix:
 			temp = []
 
 			for col in range(order[1]):
-				temp.append(-self.matrix[row][col])
+				temp.append(-self[row][col])
 
 			res.append(temp)
 		return matrix(res)
 
-	def positionOf(self, value):
+	def asList(self):
+		return self.__mat
+
+	def position(self, value):
 		row, col = self.order
 
 		for i in range(row):
 			for j in range(col):
-				if self.matrix[i][j] == value:
+				if self[i][j] == value:
 					return (i+1, j+1)
 		else:
 			raise ValueError(f"There is no Element as {value} in the Matrix.")
@@ -221,24 +237,24 @@ class matrix:
 	def minorOfValueAt(self, row, column):
 		row -= 1; column -= 1
 
-		mat = [self.matrix[i] for i in range(len(self.matrix)) if i != row]
+		mat = [self[i] for i in range(len(self.asList())) if i != row]
 
 		res = [[i[j] for j in range(len(i)) if j!=column] for i in mat]
 
 		return matrix(res)
 
 	def valueAt(self, row, column):
-		return self.matrix[row-1][column-1]
+		return self[row-1][column-1]
 
 def adjoint(matrix1):
 	"""Returns the adjoint of matrix"""
 
-	order, mat = matrix1.order, matrix1.matrix
+	order = matrix1.order
 
 	if order[0] == 2:
-		return matrix([[mat[1][1], -mat[0][1]], [-mat[1][0], mat[0][0]]])
+		return matrix([[matrix1[1][1], -matrix1[0][1]], [-matrix1[1][0], matrix1[0][0]]])
 	else:
-		res = [[((-1)**(i+j+2))*(mat[i][j])*(determinantOf(matrix1.minorOfValueAt(i+1, j+1))) for j in range(order[1])] for i in range(order[0])]
+		res = [[((-1)**(i+j+2))*(matrix1[i][j])*(determinantOf(matrix1.minorOfValueAt(i+1, j+1))) for j in range(order[1])] for i in range(order[0])]
 
 		return matrix(res)
 
@@ -266,26 +282,25 @@ def determinantOf(matrix):
 	order = matrix.order
 
 	if order[0] == order[1]:
-		mat = matrix.matrix
-
 		if order[0] == 1:
-			return mat[0][0]
+			return matrix[0][0]
 		elif order[0] == 2:
-			return (mat[1][1]*mat[0][0]) - (mat[1][0]*mat[0][1])
+			return (matrix[1][1]*matrix[0][0]) - (matrix[1][0]*matrix[0][1])
 		else:
-			M11 = mat[0][0]*(determinantOf(matrix.minorOfValueAt(1, 1)))
-			M12 = mat[0][1]*(determinantOf(matrix.minorOfValueAt(1, 2)))
-			M13 = mat[0][2]*(determinantOf(matrix.minorOfValueAt(1, 3)))
+			M11 = matrix[0][0]*(determinantOf(matrix.minorOfValueAt(1, 1)))
+			M12 = matrix[0][1]*(determinantOf(matrix.minorOfValueAt(1, 2)))
+			M13 = matrix[0][2]*(determinantOf(matrix.minorOfValueAt(1, 3)))
 
 			return M11 - M12 + M13
 	else:
 		raise ValueError(f"can only find the determinant of square matrix, not '{order[0]}x{order[1]}' matrix.")
 
 def eigenvaluesOf(matrix):
-	order = matrix.order
-	S1 = sum(matrix.primaryDiagonalValues)
+	"""Returns a list of the eigenvalues of the given matrix."""
 
-	assert order[0] in (1, 2, 3), "Maximum Order is 3x3"
+	order, S1 = matrix.order, sum(matrix.primaryDiagonalValues)
+
+	assert (order[0] in (1, 2, 3)) and (order[1] in (1, 2, 3)), "Maximum Order is 3x3"
 
 	def F(a, b, c):
 		return ((3*c/a)-((b**2)/(a**2)))/3
@@ -303,7 +318,7 @@ def eigenvaluesOf(matrix):
 
 		disc = (b**2-4*a*c)**0.5
 
-		return ((-b+disc)/(2*a), (-b-disc)/(2*a))
+		return [(-b+disc)/(2*a), (-b-disc)/(2*a)]
 	elif order[0] == 3:
 		S2 = determinantOf(matrix.minorOfValueAt(1, 1))+determinantOf(matrix.minorOfValueAt(2, 2))+determinantOf(matrix.minorOfValueAt(3, 3))
 		S3 = determinantOf(matrix)
@@ -329,21 +344,14 @@ def eigenvaluesOf(matrix):
 		h = H(g, f)
 
 		if (f==0) and (g==0) and (h==0):
-			if (d/a)>=0:
-				x = -((d/(1*a))**(1/3))
-			else:
-				x = (-d/(1*a))**(1/3)
-
+			x = -((d/(1*a))**(1/3)) if (d/a)>=0 else (-d/(1*a))**(1/3)
 			return [x]*3
 
 		elif h <= 0:
 			i = _sqrt(((g**2)/4)-h)
 			j = i**(1/3)
 			k = _acos(-(g/(2*i)))
-			L = -j
-			M = _cos(k/3)
-			N = _sqrt(3)*_sin(k/3)
-			P = -(b/(3*a))
+			L, M, N, P = -j, _cos(k/3), _sqrt(3)*_sin(k/3), -(b/(3*a))
 
 			return [round(i, 1) for i in [2*j*_cos(k/3)-(b/(3*a)), L*(M+N)+P, L*(M-N)+P]]
 
@@ -357,23 +365,24 @@ def eigenvaluesOf(matrix):
 			x2 = -(S+U)/2-(b/(3*a))+(S-U)*_sqrt(3)*0.5j
 			x3 = -(S+U)/2-(b/(3*a))-(S-U)*_sqrt(3)*0.5j
 
-			res = [x1]
-
 			def compToInt(comp):
-				str_imag = str(comp.imag)
+				imag = comp.imag
 
-				if "e" in str_imag:
+				if "e" in str(imag):
+					str_imag = str(imag)
 					e_ind = str_imag.find("e")
 					str_exp = str_imag[e_ind+1:]
 					exp = int(str_exp)
 
 					return comp.real if exp <= -5 else comp
-				else:
-					return comp
+				elif 0<abs(imag)<=1:
+					return comp.real
 
 			return [x1, compToInt(x2), compToInt(x3)]
 
-def __eigenvectorsOf(matrix):
+def eigenvectorsOf(matrix):
+	"""Generates the eigenvectors of the given matrix."""
+
 	res = []
 
 	def __isallzero(L1):
@@ -387,29 +396,46 @@ def __eigenvectorsOf(matrix):
 		return flag
 
 	evals, order = eigenvaluesOf(matrix), matrix.order
-	matobj = [_dc(matrix.matrix) for _ in range(order[0])]
+	matobj = [_dc(matrix) for _ in range(order[0])]
 
 	for eigval in evals:
-		mat = matobj.pop()
+		matrix = matobj.pop()
 
 		if isinstance(eigval, complex):
 			for i in range(order[0]):
 				for j in range(order[1]):
 					if i == j:
-						mat[i][j] = complex(mat[i][j]) - eigval
+						matrix[i][j] = complex(matrix[i][j]) - eigval
 					else:
-						mat[i][j] = complex(mat[i][j])
+						matrix[i][j] = complex(matrix[i][j])
 		else:
 			for i in range(order[0]):
 				for j in range(order[1]):
 					if i == j:
-						mat[i][j] -= int(eigval)
+						matrix[i][j] -= int(eigval)
 
-		veqns = list(map(__isallzero, mat))
-		eqn1, eqn2 = tuple([mat[i] for i in range(order[0]) if veqns[i]==False])[:2]
+		veqns = list(map(__isallzero, matrix))
+		eqn1, eqn2 = tuple([matrix[i] for i in range(order[0]) if veqns[i]==False])[:2]
 
 		if len(eqn1) == 2:
-			pass
+			llim, ulim, flag = 0, 5, True
+
+			while flag:
+				vrange = _perm([_ for _ in range(llim, ulim+1)], 2)
+
+				for a, b in vrange:
+					if (int(eqn1[0]*a + eqn1[1]*b) == 0) and (int(eqn2[0]*a + eqn2[1]*b) == 0):
+						res.append(createColumnMatrix((a, b)))
+						flag = False
+						break
+					elif (int(eqn1[0] + eqn1[1]) == 0) and (int(eqn2[0] + eqn2[1]) == 0):
+						res.append(createColumnMatrix((1, 1)))
+						flag = False
+						break
+				else:
+					llim -= 1
+					ulim += 1
+
 		elif len(eqn1) == 3:
 			x = (eqn2[2]*eqn1[1]) - (eqn2[1]*eqn1[2])
 			y = (eqn2[0]*eqn1[2]) - (eqn2[2]*eqn1[0])
@@ -433,12 +459,12 @@ def inverseOf(matrix):
 def isDiagonal(matrix):
 	"""Returns True if matrix is a 'Diagonal Matrix' else Returns False"""
 
-	order, mat = matrix.order, matrix.matrix
+	order = matrix.order
 
 	for row in range(order[0]):
 		for col in range(order[1]):
 			if row != col:
-				if mat[row][col] != 0:
+				if matrix[row][col] != 0:
 					return False
 	else:
 		return True
@@ -497,7 +523,7 @@ def isNilpotent(matrix):
 def isNull(matrix):
 	"""Returns True if matrix is a 'Null Matrix' else Returns False"""
 
-	for i in matrix.matrix:
+	for i in matrix.asList():
 		for j in i:
 			if j != 0:
 				return False
@@ -520,12 +546,12 @@ def isScalar(matrix):
 	"""Returns True if matrix is a 'Scalar Matrix' else Returns False"""
 
 	if isDiagonal(matrix):
-		order, val, mat = matrix.order, matrix.matrix[0][0], matrix.matrix
+		order, val = matrix.order, matrix[0][0]
 
 		for row in range(order[0]):
 			for col in range(order[1]):
 				if row == col:
-					if mat[row][col] != val:
+					if matrix[row][col] != val:
 						return False
 		else:
 			return True
